#!/usr/bin/perl

###
# Parse a Berkley DB file of Snort alerts, produced by
# snorta-txt2bdb and create directories of alerts
# indexed by the class C destination network of the alert.
# Also creates a separate file with counts of occurrence of each network.

###
# Load necessary libraries.
require "init.pl";


###
# Initialize input alert database.
if (! -r $ALERTDB) {
    die "Cannot read $ALERTDB. Exiting.\n$!\n";
}
if (! tie(%ALERTDB, "DB_File", $ALERTDB)) {
    die "Could not initialize input alert database $ALERTDB:\n$!\n";
}

###
# Process every entry in the input alert database
$counter = 0;
$| = 1;
%ALERTNETS = ();
while (($id, $alert) = each(%ALERTDB)) {

    # Print progress indicator as records are processed
    if ($VERBOSE && (! ($counter % 500))) {
	print ".";
    }
    $counter++;

    # Parse database entries into distinct fields.
    ($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	split(/\*/, $alert);

    # Ignore port scans that sneaked into the alert file.
    next if ($name =~ /^spp_portscan:/);

    # Obtain destination  network (Class C) of the alert. If we match purely
    # by host we would miss slight variations in addresses such as
    # those in dial-up accounts.
    next if ($dsthost eq "");
    $net = $dsthost;
    $net =~ s/.\d+$//;
    if ($net =~ /^\s*$/) {
	warn "Could not destination network from $id:$srchost\n";
	next;
    }

    # Keep track of alerts per alert day.
    $ALERTNETS{$net} .= "$id,";

}
print "\n" if ($VERBOSE);

# Group alerts by alert networks and write them to separate files
# for each network to allow them to be later imported as a spreadsheet.
# Also create a separate file with counts of occurrence of each network.
&bynet();

###
# Clean up and exit.
untie(%ALERTDB);




##
# Subroutine: Process records indexed by alert network.
sub bynet {
  my($name, $ids, @ids, $timestamp, $srchost, $srcport, $dsthost, $dstport);
  my($count);
  if (! open (SUMFILE, "> $LIBDIR/bydstnet.csv")) {
      die "Could not create $LIBDIR/bydstnet.csv:\n$!\n";
  }
  while (($net, $ids) = each(%ALERTNETS)) {

    # Obtain alert IDs associated with the alert network
    @ids = split(/,/, $ids);

    # Save all entries relating to a particular alert into one file
    $count = $#ids + 1;
    if ($VERBOSE) {
	print $net , " ($count)...\n";
    }
    print SUMFILE "$net,$count\n";
    if (! open(OUTFILE, "> $LIBDIR/bydstnet/$net.csv")) {
	warn "Could not write to $LIBDIR/bydstnet/$net:\n$!\n";
	next;
    }

    # Iterate through alert IDs for the current alert network,
    # retrieve details for each alert and write them to file.
    foreach (@ids) {
	$id = $_;
	($timestamp, $name, $srchost, $srcport, $dsthost, $dstport) =
	    split(/\*/, $ALERTDB{$id});
	print OUTFILE "$id,$timestamp,$name,$srchost,$srcport,$dsthost,$dstport\n";
    }
    close(OUTFILE);
  }
  close(SUMFILE);
}


